import cbr_content
from cbr_website_beta.content.markdown.Markdown_Parser  import markdown_parser
from cbr_website_beta.utils._for_osbot.for_OSBot_Utils  import files_names_in_folder
from osbot_utils.base_classes.Type_Safe                 import Type_Safe
from osbot_utils.utils.Files                            import files_list, file_exists, file_contents, path_combine_safe, parent_folder, folders_in_folder, folders_names_in_folder, files_names
from osbot_utils.utils.Misc                             import remove

FOLDER_NAME__WEB_PAGES       = 'web-pages'
FOLDER_NAME__BASE_FOLDER     = 'docs'

class CBR__Content__Static(Type_Safe):

    def parent_folder(self, target_file):
        base_folder = self.base_folder(target_file)
        if base_folder != FOLDER_NAME__BASE_FOLDER:
            return parent_folder(base_folder, use_full_path=False)
        return base_folder

    def base_folder(self, target_file):
        base_folder    = FOLDER_NAME__BASE_FOLDER
        file_location  = self.path_web_page(target_file)
        root_folder    = self.path_web_pages()
        file_path      = path_combine_safe(root_folder, file_location)
        if file_path:
            folder_path  = parent_folder(file_path)
            child_folder = folder_path.replace(root_folder, '')
            if child_folder != '/':
                base_folder += child_folder[1:]
        return base_folder

    def content_files(self, pattern="*"):
        return files_list(self.path_static_content(), pattern=pattern)

    def content_files__md(self):
        base_folder = self.path_static_content() + '/'
        md_files = []
        for file_path in self.content_files(pattern="*.md"):
            md_file = remove(file_path, base_folder)
            md_files.append(md_file)
        return md_files

    def file_contents(self, target_file):
        full_path = self.path_static_content_file(target_file)
        if file_exists(full_path):
            return file_contents(full_path)

    def file_contents__for__web_page(self, file_name, language='en', file_extension='md'):
        target_file           = self.path_web_page(file_name=file_name,language=language, file_extension=file_extension)
        file_contents__raw    = self.file_contents(target_file)
        if file_contents__raw:
            file_contents__parsed = self.parse_file_contents(file_contents__raw, file_extension=file_extension)
            return file_contents__parsed
        return '(no content)'


    def files(self, target):
        path_folder = self.parent_folder_of_target(target)
        #return files_in_folder(path_folder)
        return files_names_in_folder(path_folder)

    def folders(self, target):
        path_folder = self.parent_folder_of_target(target)
        return folders_names_in_folder(path_folder)

    def parse_file_contents(self, file_contents, file_extension):
        if file_extension == 'md':
            return markdown_parser.content_to_html(file_contents)
        return ''

    def parent_folder_of_target(self, target):
        relative_path = self.path_web_page(target)
        full_path     = self.path_static_content_file(relative_path)
        path_folder   = parent_folder(full_path)
        return path_folder

    def path_static_content_file(self,relative_path):
        return path_combine_safe(self.path_static_content(), relative_path)

    def path_web_page(self, file_name, language='en', file_extension='md'):
        return f'{self.path_web_pages(language)}/{file_name}.{file_extension}'

    def path_web_pages(self,language='en'):
        return f"{language}/{FOLDER_NAME__WEB_PAGES}"

    def path_to_file(self, file_location):
        return path_combine_safe(self.path_static_content(), file_location)

    def path_static_content(self):
        return cbr_content.path

cbr_content_static = CBR__Content__Static()


from flask import render_template

from cbr_website_beta.apps.chat import blueprint
from cbr_website_beta.cbr__flask.decorators.allow_annonymous import allow_anonymous
from cbr_website_beta.config.CBR__Site_Info import cbr_site_info


@blueprint.route('/history')
@allow_anonymous
def chat_history():
    from cbr_athena.llms.storage.CBR__Chats_Storage__Local import CBR__Chats_Storage__Local
    title         = "Chat - History"
    content_view  = '/llms/chat_with_llms/history.html'
    template_name = '/pages/page_with_view.html'


    cbr_chats_storage_local = CBR__Chats_Storage__Local().setup()
    chat_ids = cbr_chats_storage_local.cached_ids()

    return render_template( template_name_or_list = template_name ,
                            content_view          = content_view  ,
                            title                 = title         ,
                            chat_ids              = chat_ids      )

@blueprint.route('/view/<chat_id>')
@allow_anonymous
def chat_view__from_chat_id(chat_id):
    title             = "Chat - View past chat"
    content_view      = '/llms/chat_with_llms/view_chat_from_chat_id.html'
    template_name     = '/pages/page_with_view.html'
    url_athena        = cbr_site_info.target_athena_url()  + '/llms/chat/completion'
    url_chat_data     = cbr_site_info.target_athena_url()  + f'/llms/chat/view?chat_id={chat_id}'
    platform = "Groq (Free)"
    provider = "Meta"             # "Google"
    model    = "llama3-70b-8192"  # "gemma-7b-it"

    #provider =  "Meta"
    #model    = "llama3-70b-8192"

    return render_template( template_name_or_list = template_name ,
                            content_view          = content_view  ,
                            title                 = title         ,
                            url_athena            = url_athena    ,
                            platform              = platform      ,
                            provider              = provider      ,
                            model                 = model         ,
                            chat_id               = chat_id       ,
                            url_chat_data         = url_chat_data )


from osbot_fast_api.api.Fast_API import Fast_API


class CBR__Fast_API__Markdown(Fast_API):

    pass
    # def setup(self):
    #     #self.setup_server()
    #     super().setup()
    #     return self




import re
import xml.etree.ElementTree as etree

from markdown import Extension
from markdown.blockprocessors import BlockProcessor
from osbot_utils.base_classes.Type_Safe import Type_Safe

MERMAID_BLOCK_RE = re.compile(r'^\s*```mermaid\s*\n(.*?)\n\s*```\s*$', re.DOTALL | re.MULTILINE)

class Markdown__Ex__Mermaid(Extension):
    def extendMarkdown(self, md):
        md.parser.blockprocessors.register(Mermaid_Block_Processor(md.parser), 'mermaid', 175)


class Mermaid_Block_Processor(BlockProcessor):
    def test(self, parent, block):
        return bool(MERMAID_BLOCK_RE.search(block))

    def run(self, parent, blocks):
        block = blocks.pop(0)
        m = MERMAID_BLOCK_RE.search(block)
        if m:
            code = m.group(1)
            pre = etree.SubElement(parent, 'pre')
            pre.set('class', 'mermaid')
            pre.text = code
            script = etree.SubElement(parent, 'script')
            script.set('type', 'module')
            script.text = """
                import mermaid from 'https://cdn.jsdelivr.net/npm/mermaid@10.9.1/dist/mermaid.esm.min.mjs';
                mermaid.initialize({ startOnLoad: true });
            """


def makeExtension(**kwargs):
    return Markdown__Ex__Mermaid(**kwargs)


from cbr_website_beta.utils.Version         import version__cbr_website
from osbot_utils.base_classes.Type_Safe     import Type_Safe

DEFAULT__CONFIG__ASSETS_DIST        = '/dist'
DEFAULT__CONFIG__ASSETS_ROOT        = '/assets'
DEFAULT__CONFIG__ATHENA_URL         = "/api"
DEFAULT__CONFIG__AWS_ENABLED        = False
DEFAULT__CONFIG__CAPTURE_EXCEPTIONS = True
DEFAULT__CONFIG__CBR_LOGO           = 'cbr/cbr-logo-community.png'
DEFAULT__CONFIG__ENV                = 'LOCAL'
DEFAULT__CONFIG__LOGIN_ENABLED      = False
DEFAULT__CONFIG__GTA_ENABLED        = False


class CBR__Config__Dev(Type_Safe):
    capture_exceptions : bool = DEFAULT__CONFIG__CAPTURE_EXCEPTIONS

class CBR__Config__Website(Type_Safe):
    athena_path     : str  = DEFAULT__CONFIG__ATHENA_URL
    assets_root     : str  = DEFAULT__CONFIG__ASSETS_ROOT
    assets_dist     : str  = DEFAULT__CONFIG__ASSETS_DIST
    aws_enabled     : bool = DEFAULT__CONFIG__AWS_ENABLED
    cbr_logo        : str  = DEFAULT__CONFIG__CBR_LOGO
    env             : str  = DEFAULT__CONFIG__ENV
    login_enabled   : bool = DEFAULT__CONFIG__LOGIN_ENABLED
    gta_enabled     : bool = DEFAULT__CONFIG__GTA_ENABLED


class CBR__Config(Type_Safe):
    cbr_dev     : CBR__Config__Dev
    cbr_website : CBR__Config__Website

    def env(self):
        return self.cbr_website.env
    # def env(self):                                # todo: refactor this where this value is set via the config files
    #     return get_env('EXECUTION_ENV', 'LOCAL')


    def athena_path(self):
        return self.cbr_website.athena_path

    def assets_dist(self):
        return self.cbr_website.assets_dist

    def assets_root(self):
        return self.cbr_website.assets_root

    def aws_enabled(self):
        return self.cbr_website.aws_enabled

    def aws_disabled(self):
        return self.aws_enabled() is False

    def gta_enabled(self):
        return self.cbr_website.gta_enabled

    def cbr_logo(self):
        return self.cbr_website.cbr_logo


    def dev__capture_exceptions(self):
        return self.cbr_dev.capture_exceptions

    def login_enabled(self):
        return self.cbr_website.login_enabled

    def login_disabled(self):
        return self.login_enabled() is False


    # extra values
    def version(self):                      # todo: see if there is the best place to put this (since this value is not loaded from config file and it is already on the CBR__Site__Info object
        return version__cbr_website
    # static values                         # todo: figure out best place to store these, since they really shouldn't change
    def session_cookie_httponly(self):
        return True

    def remember_cookie_httponly(self):
        return True

    def remember_cookie_duration(self):
        return 3600                         # todo: look at increasing this value, since I think this is reason why the user session expires quite often



import frontmatter
from markdown import Markdown

from cbr_website_beta.content.markdown.Markdown__Ex__Mermaid import Markdown__Ex__Mermaid
from cbr_website_beta.content.markdown.video_extension import VideoExtension
from osbot_utils.base_classes.Type_Safe import Type_Safe
from osbot_utils.decorators.methods.cache_on_self import cache_on_self


class Markdown_Parser(Type_Safe):

    @cache_on_self
    def markdown(self):
        return Markdown(extensions=self.extensions())

    def extensions(self):
        return [Markdown__Ex__Mermaid(), VideoExtension()]

    def markdown_to_html(self, markdown_text):
        return self.markdown().convert(markdown_text)

    def content_to_html(self, content):
        _, markdown = frontmatter.parse(content)
        return  self.markdown_to_html(markdown)

markdown_parser = Markdown_Parser()


from fastapi.openapi.docs import get_swagger_ui_html

import cbr_static
from fastapi import Request, FastAPI
from starlette.responses import RedirectResponse, FileResponse, HTMLResponse
from starlette.staticfiles                                          import StaticFiles
from cbr_athena.athena__fastapi.FastAPI_Athena                      import FastAPI_Athena
from cbr_website_beta.apps.home.home_routes import running_in_aws

from cbr_website_beta.cbr__fastapi.routes.CBR__Site_Info__Routes    import CBR__Site_Info__Routes
from cbr_website_beta.cbr__fastapi__markdown.CBR__Fast_API__Markdown import CBR__Fast_API__Markdown
from cbr_website_beta.config.CBR__Config__Data                             import cbr_config
from cbr_website_beta.cbr__flask.Flask_Site                         import Flask_Site
from osbot_fast_api.api.Fast_API                                    import Fast_API
from osbot_utils.decorators.methods.cache_on_self import cache_on_self
from osbot_utils.utils.Files                                        import path_combine
from osbot_utils.utils.Http                                         import current_host_online


class CBR__Fast_API(Fast_API):

    def add_athena(self):
        cbr_athena_app = self.cbr_athena().app()
        self.app().mount("/api", cbr_athena_app)

    def add_cbr_fastapi_markdown(self):
        cbr_fastapi_markdown = self.cbr_fastapi_markdown().app()
        self.app().mount("/markdown", cbr_fastapi_markdown)

    def add_cbr_static_routes(self):
        assets_path = path_combine(cbr_static.path, 'assets')
        self.app().mount("/assets", StaticFiles(directory=assets_path, html=True), name="assets")
        return self

    def add_flask__cbr_website(self):
        flask_site = self.cbr_apps__flask_site()
        flask_app  = flask_site.app()
        path       = '/'
        self.add_flask_app(path, flask_app)
        return self

    def cbr_athena(self):
        return FastAPI_Athena().setup()

    def cbr_apps__flask_site(self):
        return Flask_Site()

    def cbr_config(self):
        return cbr_config

    def cbr_fastapi_markdown(self):
        return CBR__Fast_API__Markdown().setup()

    @cache_on_self
    def app(self):
        kwargs = {'docs_url': None }
        return FastAPI(**kwargs)

    def setup(self):
        self.setup_server()
        super().setup()
        self.add_athena              ()
        self.add_cbr_fastapi_markdown()
        self.add_cbr_static_routes   ()
        self.add_flask__cbr_website  ()             # this has to be last since it any non-resolved routes will be passed to the flask app
        return self

    def setup_routes(self):
        self.add_routes(CBR__Site_Info__Routes)
        #self.add_routes(CBR__Site_Setup__Routes)           # todo: see if we still need this

    def setup_add_root_route(self):
        app = self.app()

        @app.get("/")                                # todo: move this to a separate method
        def read_root():
            return RedirectResponse(url="/web/home")

        @app.get('/favicon.ico')                    # todo: convert the png below to .ico file (also see what are the side effects of returning a png instead of an ico)
        def favicon_ico():
            file_path = path_combine(cbr_static.path, "/assets/cbr/tcb-favicon.png")
            return FileResponse(file_path, media_type="image/png")

        @app.get('/docs', include_in_schema=False)
        async def custom_swagger_ui_html(request: Request) -> HTMLResponse:
            return self.custom_swagger_ui_html(request)

    def setup_server(self):
        #if running_in_aws():
        if current_host_online() and cbr_config.aws_enabled():
            print()
            print("####### Setting up AWS QA Server #######")
            print("#######")
            try:
                import boto3
                from osbot_utils.utils.Env import load_dotenv
                from osbot_utils.utils.Files import file_exists
                from osbot_utils.utils.Files import file_contents
                session = boto3.Session()
                s3_client = session.client('s3')
                s3_bucket = '654654216424--cbr-deploy--eu-west-1'
                s3_key = 'cbr-custom-websites/dotenv_files/cbr-site-live-qa.env'
                local_dotenv = '/tmp/cbr-site-live-qa.env'
                s3_client.download_file(s3_bucket, s3_key, local_dotenv)
                if file_exists(local_dotenv):
                    load_dotenv(local_dotenv)
                    print("####### OK: Dotenv file loaded from S3")
                else:
                    print("####### Warning: Dotenv file NOT loaded from S3")
            except Exception as error:
                print(f"####### Warning: Dotenv file NOT loaded from S3: {error}")
            print("#######")
            print("####### Setting up AWS QA Server #######")
            print()

    # todo: refactor to separate class
    def custom_swagger_ui_html(self, request: Request):
        #root       = request.scope.get("root_path")
        app         = self.app()
        title       = 'CBR' + " - Swagger UI"
        openapi_url = '/openapi.json'  # '/api/openapi.json'
        static_url  = '/assets/plugins/swagger'
        favicon     = f"{static_url}/favicon.png"

        return get_swagger_ui_html(openapi_url          = f"{openapi_url}"                     ,
                                   title                = title                                ,
                                   swagger_js_url       = f"{static_url}/swagger-ui-bundle.js" ,
                                   swagger_css_url      = f"{static_url}/swagger-ui.css"       ,
                                   swagger_favicon_url  = favicon                              ,
                                   swagger_ui_parameters = app.swagger_ui_parameters           )

cbr_fast_api = CBR__Fast_API().setup()


from cbr_website_beta.config.CBR__Config__Data                         import cbr_config
from cbr_website_beta.config.CBR__Config__Data import cbr_config_active
from cbr_website_beta.config.CBR__Site_Info                     import CBR__Site_Info, cbr_site_info
from cbr_website_beta.utils.health_checks.CBR__Health_Checks    import CBR__Health_Checks, cbr_health_checks
from osbot_fast_api.api.Fast_API_Routes                         import Fast_API_Routes

ROUTE_PATH__SITE_INFO     = 'site_info'
EXPECTED_SITE_INFO_ROUTES = ['/cbr-config-active', '/cbr-site-info', '/health-checks']

class CBR__Site_Info__Routes(Fast_API_Routes):

    tag : str = ROUTE_PATH__SITE_INFO

    def cbr_config_active(self):
        return cbr_config_active.json()

    def cbr_site_info(self):
        return cbr_site_info.data()

    def health_checks(self):
        return cbr_health_checks.execute_health_checks()

    def setup_routes(self):
        self.add_route_get(self.cbr_config_active)
        self.add_route_get(self.cbr_site_info    )
        self.add_route_get(self.health_checks    )


from flask import render_template

from cbr_website_beta.apps.docs                                 import blueprint
from cbr_website_beta.cbr__flask.decorators.allow_annonymous    import allow_anonymous
from cbr_website_beta.content.CBR__Content__Static              import cbr_content_static

EXPECTED_ROUTES__MINERVA = [ '/docs']

@blueprint.route('')
#@blueprint.route('/')                  # this is not working with the route ''
@blueprint.route('/<path:path>')
@allow_anonymous
def minerva_root(path='/'):
    content_view    = 'docs/index.html'
    template_name = '/pages/page_with_view.html'
    title         = 'Documentation'
    if path == '' or path == '/':
        path = 'index'
    content       = cbr_content_static.file_contents__for__web_page(path)
    base_folder   = cbr_content_static.base_folder                 (path)
    parent_folder = cbr_content_static.parent_folder               (path)
    folders       = cbr_content_static.folders                     (path)
    files         = cbr_content_static.files                       (path)
    return render_template(template_name_or_list = template_name,
                           title                 =  title        ,
                           content_view          = content_view  ,
                           path                  = path          ,
                           content               = content       ,
                           files                 = files         ,
                           folders               = folders       ,
                           base_folder           = base_folder   ,
                           parent_folder         = parent_folder )




import cbr_website_beta
from cbr_website_beta.config.CBR__Config__Active    import CBR__Config__Active
from osbot_utils.utils.Misc                         import date_time_now
from osbot_utils.utils.Status                       import status_ok, status_error
from osbot_utils.utils.Toml                         import toml_dict_from_file
from osbot_utils.utils.Env                          import get_env
from cbr_website_beta.config.CBR__Config            import CBR__Config
from osbot_utils.base_classes.Type_Safe             import Type_Safe
from osbot_utils.utils.Files                        import files_names, folder_files, path_combine_safe

ENV_VAR_NAME__CBR__CONFIG_FILE     = 'CBR__CONFIG_FILE'
DEFAULT__CBR__CONFIG_FILE          = 'cbr-website.community.toml'
FOLDER__CBR__CONFIG_FILES          = './config'

class CBR__Config__Data(Type_Safe):
    #cbr_config : CBR__Config

    # def __init__(self):
    #     super().__init__()
    #     load_dotenv()

    def current_config(self):
        return self.load_config_file(self.current_config_file_name())

    def config_files(self):
        return files_names(folder_files(self.path_config_files(), pattern="*.toml"))

    def current_config_file_name(self):
        return get_env(ENV_VAR_NAME__CBR__CONFIG_FILE) or DEFAULT__CBR__CONFIG_FILE

    def load_config_file(self, config_file_name):
        try:
            path_config_file = self.path_config_file(config_file_name)
            config_data      = toml_dict_from_file(path_config_file)
            cbr_config       = CBR__Config(**config_data)
            return status_ok(data=cbr_config)
        except Exception as error:
            return status_error(message='In CBR__Config failed to load config file', error=f'{error}', data={})

    def create_cbr_config_active__from_current_config_file(self):
        config_file_name = self.current_config_file_name()
        return self.create_cbr_config_active(config_file_name)

    def create_cbr_config_active(self, config_file_name):
        load_cbr_config     = self.load_config_file(config_file_name)
        file_loaded_at      = date_time_now()
        if load_cbr_config.get('status') == 'ok':
            status      = "config file loaded ok"
            cbr_config_data  = self.load_config_file(config_file_name).get('data')
            kwargs      = dict(cbr_config       = cbr_config_data      ,
                               config_file_name = config_file_name,
                               file_loaded_at   = file_loaded_at  ,
                               status           = status          )
            return CBR__Config__Active(**kwargs)
        else:
            status = f"error: config file failed to load: {load_cbr_config.get('error')}"
            cbr_config_data  = CBR__Config()                                                # use default values
            kwargs           = dict(cbr_config   = cbr_config_data ,
                               config_file_name  = config_file_name,
                               file_loaded_at    = file_loaded_at  ,
                               status            = status          )
            return CBR__Config__Active(**kwargs)


    def path_config_file(self, file_name):
        return path_combine_safe(self.path_config_files(), file_name)

    def path_config_files(self):
        return path_combine_safe(cbr_website_beta.path, FOLDER__CBR__CONFIG_FILES)

cbr_config_active = CBR__Config__Data().create_cbr_config_active__from_current_config_file()
cbr_config = cbr_config_active.cbr_config


import traceback

from flask import render_template, Flask
from werkzeug.exceptions import NotFound

from cbr_website_beta._cbr_shared.dynamo_db.DyDB__CBR_Logging import DyDB__CBR_Logging
from cbr_website_beta.config.CBR__Config__Data import cbr_config
from cbr_website_beta.cbr__flask.decorators.allow_annonymous      import allow_anonymous
from osbot_utils.utils.Dev import pprint
from osbot_utils.utils.Objects import type_full_name, obj_info


def register_error_handling(app: Flask):

    dydb_cbr_logging = DyDB__CBR_Logging()

    @app.errorhandler(Exception)
    def internal_server_error(exception):
        if type(exception) is NotFound:
            return render_template('home/page-404.html', error=str(exception)), 404
        if cbr_config.dev__capture_exceptions():
            dydb_cbr_logging.log_exception(exception)
        return render_template('home/page-500.html', error=str(type(exception))), 500

    @app.route('/web/raise_exception')
    @allow_anonymous
    def cause_error():
        # This will cause an error and trigger the error handler
        return 1 / 0


from cbr_website_beta.config.CBR__Config import CBR__Config
from osbot_utils.base_classes.Type_Safe import Type_Safe


class CBR__Config__Active(Type_Safe):
    cbr_config       : CBR__Config
    config_file_name : str
    file_loaded_at   : str
    status           : str


from unittest import TestCase

from cbr_website_beta.cbr__fastapi.CBR__Fast_API__Client import CBR__Fast_API__Client


class TestCase__CBR__Website(TestCase):

    @classmethod
    def setUpClass(cls) -> None:
        cls.client       = CBR__Fast_API__Client().setup()
        cls.cbr_fast_api = cls.client.cbr_fast_api
        cls.cbr_athena   = cls.cbr_fast_api.cbr_athena()






from os import environ

from osbot_aws.aws.sts.STS import STS

import cbr_athena

import cbr_static

import cbr_website_beta
from cbr_athena.aws.s3.S3_DB_Base import S3_DB_Base
from cbr_website_beta.config.CBR__Config__Data                     import cbr_config
from osbot_utils.utils.Env                                  import get_env, is_env_var_set, env__old_pwd__remove, env__pwd
from cbr_athena.utils.Version                               import version__cbr_athena
from cbr_static.utils.Version                               import Version as Version__cbr_static
from osbot_aws.AWS_Config                                   import aws_config
from osbot_fast_api.utils.Version                           import Version as Version__osbot_fast_api
from osbot_utils.utils.Status                               import status_error
from osbot_utils.utils.Version                              import Version as Version__osbot_utils
from osbot_utils.base_classes.Type_Safe                     import Type_Safe
from cbr_website_beta.utils.Version import version, version__cbr_website, version__cbr_content
from osbot_utils.decorators.methods.cache_on_self           import cache_on_self


class CBR__Site_Info(Type_Safe):

    def aws_configured(self):
        with aws_config as _:
            if _.aws_access_key_id():
                if _.aws_secret_access_key():
                    if _.region_name():
                        return True
        return False

    def data(self):
        try:
            return dict(aws      = self.aws      (),
                        dates    = self.dates    (),
                        env_vars = self.env_vars (),
                        paths    = self.paths    (),
                        urls     = self.urls     (),
                        versions = self.versions ())
        except Exception as error:
            return status_error(message="error in CBR__Site_Info.data", error=f'{error}')

    def aws(self):
        caller_identity = STS().caller_identity()
        return dict(caller_identity       = caller_identity          ,
                    region                = aws_config.region_name() ,
                    s3_bucket__s3_db_base = S3_DB_Base().s3_bucket() )

    def dates(self):
        return dict(cbr_site_published_at = get_env('CBR__SITE__PUBLISHED_AT', ''))

    def env_vars(self):
        return dict(status = self.env_vars__status(),
                    values = self.env_vars__values())

    def env_vars__status(self):
        var_names = ['OPEN_AI__API_KEY', 'IP_DATA__API_KEY', 'OPEN_ROUTER_API_KEY', 'GROQ_API_KEY', 'COGNITO_USER_POOL_ID']
        status = {}
        for var_name in var_names:
            status[var_name] = is_env_var_set(var_name)
        return status

    def env_vars__values(self):
        var_names = ['CBR__CONFIG_FILE', 'EXECUTION_ENV', 'PORT', 'S3_DEV__VERSION' , 'AWS_LWA_INVOKE_MODE']
        values = {}
        for var_name in var_names:
            values[var_name] = get_env(var_name)
        return values

    def paths(self):
        return dict(cbr_athena       = env__old_pwd__remove(cbr_athena      .path),
                    cbr_static       = env__old_pwd__remove(cbr_static      .path),
                    cbr_website_beta = env__old_pwd__remove(cbr_website_beta.path),
                    pwd              = env__pwd()                                )


    def target_athena_url(self):        # todo: refactor out once new setup is stable
        return cbr_config.athena_path()

    def url_athena__internal(self):
        port        = self.cbr_host__port()
        athena_path = cbr_config.athena_path()
        if athena_path.startswith('http'):
            return athena_path
        if port:
            return f'http://localhost:{port}{athena_path}'

    def urls(self):
        return dict(url_athena           = self.target_athena_url   (),
                    url_athena__internal = self.url_athena__internal(),
                    url_assets_dist      = cbr_config.assets_dist     ,
                    url_assets_root      = cbr_config.assets_root     )

    @cache_on_self
    def version(self):
        return version

    def versions(self):
        cbr   = dict(cbr_athena     = version__cbr_athena              ,
                     cbr_content    = version__cbr_content             ,
                     cbr_website    = version__cbr_website             ,
                     cbr_static     = Version__cbr_static    ().value())       # todo create: version__cbr_static
        osbot = dict(osbot_fast_api = Version__osbot_fast_api().value(),        # todo create: version__osbot_fast_api
                     osbot_utils    = Version__osbot_utils   ().value())       # todo create: version__osbot_utils

        return dict(cbr      = cbr   ,
                    osbot    = osbot )


    # individual values
    def cbr_host__port(self):
        return get_env('PORT')


cbr_site_info = CBR__Site_Info()


from osbot_utils.base_classes.Type_Safe import Type_Safe


class LLMs__Platforms(Type_Safe):

    def model_options(self):
        return { 'Groq (Free)'       : { 'Meta'         : { 'LLaMA3 8b'          : 'llama3-8b-8192'                            ,
                                                            'LLaMA3 70b'         : 'llama3-70b-8192'                            },
                                         'Mistral'      : { 'Mixtral 8x7b'       : 'mixtral-8x7b-32768'                        },
                                         'Google'       : { 'Gemma 7b'           : 'gemma-7b-it'                               }},

                 'Open Router (Free)': { 'Google'       : { 'Gemma 7b'           : 'google/gemma-7b-it:free'                   ,
                                                            'Gemma 9b'           : 'google/gemma-2-9b-it:free'                 },
                                         'Meta'         : { 'LLaMA3 8b'          : 'meta-llama/llama-3-8b-instruct:free'       },
                                         'Microsoft'    : { 'Phi-3 Medium 128K'  : 'microsoft/phi-3-medium-128k-instruct:free' ,
                                                            'Phi-3 Mini 128K'    : 'microsoft/phi-3-mini-128k-instruct:free'   },
                                         'Nous Research': { 'Capybara 7b'        : 'nousresearch/nous-capybara-7b:free'        },
                                         'Open Chat'    : { 'Openchat 7b'        : 'openchat/openchat-7b:free'                 },
                                         'Gryphe'       : { 'Mythomist 7b'       : 'gryphe/mythomist-7b:free'                  },
                                         'Wild 7B'      : { 'Toppy M 7b'         : 'undi95/toppy-m-7b:free'                    },
                                         'Hugging Face' : { 'Zephyr 7b'          : 'huggingfaceh4/zephyr-7b-beta:free'         },
                                         'Mistral'      : { 'Mistral 7b Instruct': 'mistralai/mistral-7b-instruct:free'        }},
                'Open Router (Paid)' : { 'Qwen'         : { 'Qwen 2 72B'         : 'qwen/qwen-2-72b-instruct'                  },
                                         'Anthropic'    : { 'Claude 3.5 Sonnet'  : 'anthropic/claude-3.5-sonnet'               },
                                         'Gryphe'       : { 'MythoMax 13b'       : 'gryphe/mythomax-l2-13b'                    },
                                         'Meta'         : { 'Llama3 70b Instruct': 'meta-llama/llama-3-70b-instruct'           },
                                         'Microsoft'    : { 'WizardLM-2 8x22b'   : 'microsoft/wizardlm-2-8x22b'                },
                                         'Nvidia'       : { 'Nemotron-4 340b'    : 'nvidia/nemotron-4-340b-instruct'           },
                                         'Google'       : { 'Gemini Flash 1.5'   : 'google/gemini-flash-1.5'                   ,
                                                            'Gemma 9b'           : 'google/gemma-2-9b-it'                      },
                                         'Sao10K'       : { 'Llama3 Euryale 70b' : 'sao10k/l3-euryale-70b'                     }},

                 'Ollama (Local)'    : { 'Meta'         : { 'LLaMA3 8b'          : 'llama3'                                     },
                                         'Microsoft'    : { 'Phi 3b (Mini)'      : 'phi3'                                       },
                                         'Google'       : { 'Gemma 7b'           : 'gemma'                                     }},

                 'OpenAI (Paid)'     : { 'OpenAI'       : { 'GPT 4o'             : 'gpt-4o'                                    ,
                                                            'GPT 4 Turbo'        : 'gpt-4-turbo'                               ,
                                                            'GPT 3.5 Turbo'      : 'gpt-3.5-turbo'                             }}}


